/***********************************************************************************************************************
 *
 * blueBill Resources - open source birding
 * Copyright (C) 2009-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://bluebill.tidalwave.it
 * SCM: https://kenai.com/hg/bluebill~resources-src
 *
 **********************************************************************************************************************/
package it.tidalwave.bluebill.taxonomy.birds.clements;

import javax.annotation.Nonnull;
import java.io.File;
import it.tidalwave.bluebill.taxonomy.birds.BirdTaxonomyImporter;
import it.tidalwave.bluebill.taxonomy.birds.DefaultTranslator;
import it.tidalwave.bluebill.taxonomy.birds.GeneratorSupport;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id: $
 *
 **********************************************************************************************************************/
public class Clements2008Generator extends GeneratorSupport
  {
    public static final String TAXONOMY_NAME = "Clements 6.3.2";
    public static final String TAXONOMY_FILE_NAME = "Clements-6.3.2";

    public static void main (final @Nonnull String ... args)
      throws Exception
      {
        new Clements2008Generator(args[0]).execute();
      }

    public Clements2008Generator (final @Nonnull String baseDir)
      {
        super(baseDir, TAXONOMY_NAME, TAXONOMY_FILE_NAME);
      }

    @Nonnull
    public BirdTaxonomyImporter run()
      throws Exception
      {
        final Clements2008Importer importer = new Clements2008Importer();
        importer.setTaxonTranslator(new DefaultTranslator(Clements2008Importer.class, "Clements2008ToCatalogueOfLifeTranslations.config"));
        importer.setSourceResourceName("Clements Checklist 6.3.2 December 2008.csv.gz");
        importer.setIdentifierMapFile(new File(resourcesFolder, "TaxonIdMap.txt.gz"));
        importer.run(repository, taxonomyName);

        return importer;
      }
  }
